@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2022 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -72,6 +72,7 @@ REHex::StringPanel::StringPanel(wxWindow *parent, SharedDocumentPointer &documen
 	ignore_cjk(false),
 	update_needed(false),
 	threads_exit(true),
+	timer(this, wxID_ANY),
 	threads_pause(false),
 	spawned_threads(0),
 	running_threads(0),
@@ -153,8 +154,6 @@ REHex::StringPanel::StringPanel(wxWindow *parent, SharedDocumentPointer &documen
 	
 	mark_dirty(0, document->buffer_length());
 	
-	timer = new wxTimer(this, wxID_ANY);
-	
 	start_threads();
 }
 
@@ -628,19 +627,22 @@ void REHex::StringPanel::thread_flush(ByteRangeSet *set_ranges, ByteRangeSet *cl
 	{
 		std::lock_guard<std::mutex> sl(strings_lock);
 		
-		off_t processed_total = sum_clean_bytes();
-		size_t size_hint = (double)(strings.size()) * ((double)(document->buffer_length()) / (double)(processed_total));
-		
-		if(size_hint > MAX_STRINGS)
+		if(!set_ranges->empty())
 		{
-			size_hint = MAX_STRINGS;
+			off_t processed_total = sum_clean_bytes();
+			size_t size_hint = (double)(strings.size()) * ((double)(document->buffer_length()) / (double)(processed_total));
+			
+			if(size_hint > MAX_STRINGS)
+			{
+				size_hint = MAX_STRINGS;
+			}
+			
+			strings.set_ranges(set_ranges->begin(), set_ranges->end(), size_hint);
+			set_ranges->clear_all();
+			
+			update_needed = true;
 		}
 		
-		strings.set_ranges(set_ranges->begin(), set_ranges->end(), size_hint);
-		set_ranges->clear_all();
-		
-		update_needed = true;
-		
 		if(strings.size() >= MAX_STRINGS)
 		{
 			/* Reached the string limit, start spinning down. */
@@ -700,9 +702,9 @@ void REHex::StringPanel::start_threads()
 				++running_threads;
 			}
 			
-			if(!timer->IsRunning())
+			if(!timer.IsRunning())
 			{
-				timer->Start(100, wxTIMER_CONTINUOUS);
+				timer.Start(100, wxTIMER_CONTINUOUS);
 			}
 		#if 0
 		}
@@ -727,7 +729,7 @@ void REHex::StringPanel::stop_threads()
 	spinner->Stop();
 	spinner->Hide();
 	
-	timer->Stop();
+	timer.Stop();
 	
 	{
 		std::lock_guard<std::mutex> pl(pause_lock);
