@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2017-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2017-2022 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -57,7 +57,7 @@ namespace REHex
 	{
 		public:
 			Tab(wxWindow *parent);
-			Tab(wxWindow *parent, const std::string &filename);
+			Tab(wxWindow *parent, SharedDocumentPointer &document);
 			
 			virtual ~Tab();
 			
