@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2017-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2017-2022 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -136,6 +136,14 @@ std::string REHex::Document::get_title()
 	return title;
 }
 
+void REHex::Document::set_title(const std::string &title)
+{
+	this->title = title;
+	
+	DocumentTitleEvent document_title_event(this, title);
+	ProcessEvent(document_title_event);
+}
+
 std::string REHex::Document::get_filename()
 {
 	return filename;
@@ -480,7 +488,7 @@ int REHex::Document::overwrite_text(off_t offset, const std::string &utf8_text,
 	
 	int ret_flags = WRITE_TEXT_OK;
 	
-	CharacterEncoderIconv utf8_encoder("UTF-8", 1);
+	CharacterEncoderIconv utf8_encoder("UTF-8", 1, true);
 	
 	for(off_t utf8_off = 0, write_pos = offset; utf8_off < (off_t)(utf8_text.size());)
 	{
@@ -555,7 +563,7 @@ int REHex::Document::insert_text(off_t offset, const std::string &utf8_text, off
 	
 	int ret_flags = WRITE_TEXT_OK;
 	
-	CharacterEncoderIconv utf8_encoder("UTF-8", 1);
+	CharacterEncoderIconv utf8_encoder("UTF-8", 1, true);
 	
 	std::string data_type;
 	const CharacterEncoder *encoder;
@@ -636,7 +644,7 @@ int REHex::Document::replace_text(off_t offset, off_t old_data_length, const std
 	
 	int ret_flags = WRITE_TEXT_OK;
 	
-	CharacterEncoderIconv utf8_encoder("UTF-8", 1);
+	CharacterEncoderIconv utf8_encoder("UTF-8", 1, true);
 	
 	const CharacterEncoder *encoder = get_text_encoder(offset);
 	assert(encoder != NULL);
@@ -835,7 +843,7 @@ bool REHex::Document::set_data_type(off_t offset, off_t length, const std::strin
 			_raise_types_changed();
 		},
 		
-		[this]()
+		[]()
 		{
 			/* Data type changes are undone implicitly. */
 		});
@@ -886,7 +894,7 @@ bool REHex::Document::set_virt_mapping(off_t real_offset, off_t virt_offset, off
 			_raise_mappings_changed();
 		},
 		
-		[this]()
+		[]()
 		{
 			/* Address mapping changes are undone implicitly. */
 		});
@@ -944,7 +952,7 @@ void REHex::Document::clear_virt_mapping_r(off_t real_offset, off_t length)
 			_raise_mappings_changed();
 		},
 		
-		[this]()
+		[]()
 		{
 			/* Address mapping changes are undone implicitly. */
 		});
@@ -1000,7 +1008,7 @@ void REHex::Document::clear_virt_mapping_v(off_t virt_offset, off_t length)
 			_raise_mappings_changed();
 		},
 		
-		[this]()
+		[]()
 		{
 			/* Address mapping changes are undone implicitly. */
 		});
@@ -1206,6 +1214,18 @@ const char *REHex::Document::redo_desc()
 	}
 }
 
+void REHex::Document::reset_to_clean()
+{
+	current_seq = 0;
+	buffer_seq = 0;
+	saved_seq = 0;
+	data_seq.set_range(0, buffer->length(), 0);
+	
+	undo_stack.clear();
+	redo_stack.clear();
+	_raise_undo_update();
+}
+
 void REHex::Document::transact_begin(const std::string &desc)
 {
 	if(undo_stack.empty() || undo_stack.back().complete)
@@ -1545,6 +1565,17 @@ json_t *REHex::Document::_dump_metadata(bool& has_data)
 		return NULL;
 	}
 	
+	if(json_object_set_new(root, "write_protect", json_boolean(write_protect)) == -1)
+	{
+		json_decref(root);
+		return NULL;
+	}
+	
+	if(write_protect)
+	{
+		has_data = true;
+	}
+	
 	json_t *comments = json_array();
 	if(json_object_set_new(root, "comments", comments) == -1)
 	{
@@ -1784,6 +1815,9 @@ void REHex::Document::_load_metadata(const std::string &filename)
 	types = _load_types(meta, buffer_length());
 	std::tie(real_to_virt_segs, virt_to_real_segs) = _load_virt_mappings(meta, buffer_length());
 	
+	json_t *write_protect = json_object_get(meta, "write_protect");
+	set_write_protect(json_is_true(write_protect));
+	
 	json_decref(meta);
 }
 
