@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2017-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2017-2022 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -137,6 +137,11 @@ namespace REHex {
 			*/
 			std::string get_title();
 			
+			/**
+			 * @brief Set the user-visible title of the document.
+			*/
+			void set_title(const std::string &title);
+			
 			/**
 			 * @brief Get the filename of the document, or an empty string if there is no backing file.
 			*/
@@ -278,6 +283,11 @@ namespace REHex {
 			*/
 			const char *redo_desc();
 			
+			/**
+			 * @brief Clear the undo/redo history and mark the document as clean.
+			*/
+			void reset_to_clean();
+			
 		#ifndef UNIT_TEST
 		private:
 		#endif
@@ -343,7 +353,6 @@ namespace REHex {
 			std::string title;
 			
 			off_t cpos_off{0};
-			bool insert_mode{false};
 			
 			enum CursorState cursor_state;
 			
