@@ -18,6 +18,7 @@
 #include "../src/platform.hpp"
 #include <gtest/gtest.h>
 #include <wx/app.h>
+#include <wx/fontutil.h>
 #include <wx/init.h>
 
 #include "../src/App.hpp"
@@ -40,6 +41,7 @@ struct Cleanup
 	{
 		delete REHex::active_palette;
 		delete wxGetApp().recent_files;
+		delete wxGetApp().settings;
 		delete wxGetApp().config;
 		delete wxGetApp().console;
 	}
@@ -53,10 +55,19 @@ int main(int argc, char **argv)
 	wxInitializer wxinit;
 	
 	wxFont default_font(wxFontInfo().Family(wxFONTFAMILY_MODERN));
+	
+	#ifdef __APPLE__
+	/* wxWidgets 3.1 on Mac returns an empty string from wxFont::GetFaceName() at this
+	 * point for whatever reason, but it works fine later on....
+	*/
+	app->set_font_name(default_font.GetNativeFontInfo()->GetFaceName().ToStdString());
+	#else
 	app->set_font_name(default_font.GetFaceName().ToStdString());
+	#endif
 	
 	app->console = new REHex::ConsoleBuffer();
 	app->config = new wxConfig("REHex-qwertyuiop"); /* Should be a name that won't load anything. */
+	app->settings = new REHex::AppSettings();
 	app->recent_files = new wxFileHistory();
 	
 	app->_test_setup_hooks(REHex::App::SetupPhase::EARLY);
